<?php

namespace Drupal\commerce_giftcard\Plugin\Validation\Constraint;

use Drupal\commerce_giftcard\Entity\GiftcardTransactionInterface;
use Drupal\commerce_price\Plugin\Field\FieldType\PriceItem;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates the GiftcardTransactionValidAmount constraint.
 */
class GiftcardTransactionValidAmountValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint) {
    $item = $value->get(0);
    assert($item instanceof PriceItem);
    $transaction = $value->getEntity();
    assert($transaction instanceof GiftcardTransactionInterface);

    // Do not validate an already saved transaction.
    if (!$transaction->isNew() && !$item->isEmpty()) {
      return;
    }

    $gift_card = $transaction->getGiftCard();
    if (!$gift_card) {
      return;
    }

    $price = $item->toPrice();
    if ($price) {
      if ($price->getCurrencyCode() != $gift_card->getBalance()->getCurrencyCode()) {
        $this->context->buildViolation($constraint->currencyMessage, [
          '%transaction_currency' => $price->getCurrencyCode(),
          '%giftcard_currency' => $gift_card->getBalance()->getCurrencyCode(),
        ])
          ->atPath('0.currency_code')
          ->setInvalidValue($price->getNumber())
          ->addViolation();
      }
      elseif ($price->isNegative() && $price->multiply('-1')->greaterThan($gift_card->getBalance())) {
        $this->context->buildViolation($constraint->message)
          ->atPath('0.number')
          ->setInvalidValue($price->getNumber())
          ->addViolation();
      }
    }
  }

}
